/*
 * (c) 1997 Eric Sven Ristad.  All rights reserved.
 *
 * $Id: L.h,v 1.13 1997/10/27 01:34:44 libpa Exp $
 *
 */

#ifndef L_INCLUDED
#define L_INCLUDED

#include <math.h>
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>

/* Description.
 *  Fixed-point logarithmic representation of probability values.
 *  Requires: L_BASE, L_LNBASE, L_LGBASE, L_ROUNDUP, L_TABLE_SIZE, L_delta()
 *
 */

#define L_UNITY       0                        /* unit probability */
#define L_ZERO        ((unsigned)UINT_MAX)     /* zero probability */
#ifdef SIGUSR1
 #define L_SIGNAL     SIGUSR1
#else
 #define L_SIGNAL     SIGFPE
#endif

typedef unsigned char  Lchar_t;
typedef unsigned short Lshort_t;
typedef unsigned int   Lint_t;

extern unsigned long   L_Max[];
extern int    kill(pid_t, int);
extern pid_t  getpid(void);

/************************************************************
 * Constructors/Coercion
 ************************************************************/

#define L2double(l)     (((l) >= (-(DBL_MIN_EXP)/L_LGBASE)) ? 0 : pow(L_BASE, -(double)(l)))
#define double2L(x)     (((0 < (x)) && ((x) <= 1)) ? \
			 pr2L(x) : (((x) == 0) ? L_ZERO : L_raise()))

#define L2nats(l)       ((double)(l) * L_LNBASE)
#define nats2L(n)       (L_truncate((double)(n) / L_LNBASE))

#define L2bits(l)       ((double)(l) * L_LGBASE)
#define bits2L(n)       (L_truncate((double)(n) / L_LGBASE))

/************************************************************
 * Predicates
 ************************************************************/

#define L_is_valid(l)        ((l) >= 0)
#define L_compare(l1,l2)     ((int)((l2) - (l1)))
#define L_are_equal(l1,l2)   ((l1) == (l2))
#define L_is_greater(l1,l2)  ((l1) < (l2))
#define L_is_less(l1,l2)     ((l1) > (l2))
#define L_is_geq(l1,l2)      ((l1) <= (l2))
#define L_is_leq(l1,l2)      ((l1) >= (l2))

/************************************************************
 * Arithmetic
 ************************************************************/

#ifndef NDEBUG

 #define L_multiply(l1,l2)   (((L_Max[sizeof(l1)]-(l1) >= (l2)) ||  \
			       (L_Max[sizeof(l2)]-(l2) >= (l1)))    \
			      ? (l1)+(l2) : L_raise())
 #define L_divide(l1,l2)     (((l1) >= (l2)) ? (l1)-(l2) : L_raise())
 #define L_add(l1,l2)        (((l1) < (l2)) ? \
			      (((l1) >= L_delta((l2)-(l1))) ? \
			       (l1) - L_delta((l2)-(l1)) : L_raise()) : \
			      (((l2) >= L_delta((l1)-(l2))) ? \
			       (l2) - L_delta((l1)-(l2)) : L_raise()))
 #define L_complement(l)     (((l) > 0) ? pr2L(1-L2double(l)) : L_ZERO)
			       
#else

 #define L_multiply(l1,l2)   ((l1)+(l2))
 #define L_divide(l1,l2)     ((l1)-(l2))
 #define L_add(l1,l2)        (((l1) < (l2)) ? ((l1) - L_delta((l2)-(l1))) : \
			                      ((l2) - L_delta((l1)-(l2))))
 #define L_complement(l)     (((l) > 0) ? pr2L(1-L2double(l)) : L_ZERO)

#endif

/************************************************************
 * Exception Handling
 ************************************************************/

#define L_raise()            (kill(getpid(), L_SIGNAL))
#define L_signal(handler_p)  (signal(L_SIGNAL, (handler_p)))

/************************************************************
 * Internal
 ************************************************************/

#define pr2L(x)              (-floor((log(x)/L_LNBASE)+L_ROUNDUP))
#define L_truncate(l)        (floor((l)+0.5))
#define L_delta_raw(n)       (log1p(pow(L_BASE,-(n))) / L_LNBASE)

#endif


