/*
 * (c) 1996 by Eric Sven Ristad and Peter N. Yianilos.  All rights reserved.
 *
 * $Id: balanced.h,v 1.6 1997/06/05 21:44:38 ristad Exp $
 * 
 */

#ifndef BALANCED_INCLUDED
#define BALANCED_INCLUDED

#include <stdio.h>
#include <limits.h>
#include <float.h>
#include "common.h"

#define BALANCED_RADIX        (FLT_RADIX)
#define BALANCED_PRECISION    (FLT_MANT_DIG)
#define BALANCED_MANT_LOW      0.5
#define BALANCED_MANT_HIGH     1.0
#define BALANCED_EPSILON      (FLT_EPSILON)
#define BALANCED_MIN_EXP       INT_MIN
#define BALANCED_MAX_EXP       INT_MAX

#ifndef M_LN2
#define M_LN2                  6.9314718055994530942E-1
#endif
#ifndef M_LN10
#define M_LN10                 2.3025850929940456840E0
#endif

typedef struct balanced_tag
{
  float  sig;
  int    exp;
} balanced_t;

/************************************************************
 * Constructors
 ************************************************************/

balanced_t  double2balanced(double x);
double	    balanced2double(balanced_t bx);

balanced_t  balanced_zero(void);
balanced_t  balanced_unity(void);

/************************************************************
 * Predicates 
 ************************************************************/ 

boolean_t   balanced_is_zero(balanced_t bx);
boolean_t   balanced_is_unity(balanced_t bx);
boolean_t   balanced_is_pr(balanced_t bx);
boolean_t   balanced_is_valid(balanced_t bx);

/************************************************************
 * Comparision
 ************************************************************/ 

#define     balanced_are_equal(bx,by)   (((bx).sig == (by).sig) && ((bx).exp == (by).exp))
int	    balanced_compare(balanced_t bx, balanced_t by, balanced_t tolerance);
int	    balanced_exact_compare(balanced_t bx, balanced_t by);

/************************************************************
 * Arithmetic
 ************************************************************/ 

balanced_t  balanced_add(balanced_t bx, balanced_t by);
balanced_t  balanced_subtract(balanced_t bx, balanced_t by);
balanced_t  balanced_multiply(balanced_t bx, balanced_t by);
balanced_t  balanced_divide(balanced_t bx, balanced_t by);
balanced_t  balanced_magnitude(balanced_t bx);

/************************************************************
 * Vector Arithmetic
 ************************************************************/ 

void        balanced_inner_step(balanced_t *bx_p, const balanced_t *by_p, 
				const balanced_t *bz_p);
            /* Effect: bx += by * bz */

/************************************************************
 * Transcendental
 ************************************************************/ 

balanced_t  balanced_power(balanced_t bx, double n);
balanced_t  balanced_exp(double x);
double	    balanced_log(balanced_t bx);
double	    balanced_log2(balanced_t bx);
#define     balanced_log10(bx)          (balanced_log(bx)/M_LN10)

/************************************************************
 * Input/Output
 ************************************************************/ 

int         balanced_fprintf(balanced_t bx, FILE *stream);
            /* Returns: number of objects printed on stream */
int         balanced_printf(balanced_t bx);


#endif	/* BALANCED_INCLUDED */






