/*
 * (c) 1996 Eric Sven Ristad.  All rights reserved.
 *
 * $Id: hash.class.h,v 1.5 1998/02/18 07:54:08 libpa Exp libpa $
 *
 */

#ifndef HASH_CLASS_INCLUDED
#define HASH_CLASS_INCLUDED

/* Description.
 * hash_class_t defines a class of hash_table_t that share the same
 * operating parameters, including key comparision and hashing.
 */

typedef struct hash_class_tag
{
  const double  maxload;    /* upper bound to trigger upsize   */
  const double  minload;    /* lower bound to trigger downsize */
                            /* warning: maxload / expansion_factor > minload */
                            /* warning: minload * expansion_factor < maxload */

  double  collisions;       /* number of collisions in entire class */
  double  redundancies;     /* number of redundant hash values */
  double  probes;           /* total number of lookups */

  const unsigned  capacity_count;  /* number of capacities provided */
  const unsigned *capacity;        /* index2capacity[] table        */

  int       (*compare)(const void *lkey_p, const void *rkey_p, const void *parameters);
             /* Returns: zero iff both keys are equal               */
             /* Note: this function is called in the innermost loop */
  unsigned  (*primary_hash)(const void *key_p, const void *parameters);
             /* Returns: primary hash value for *key_p              */
  unsigned  (*secondary_hash)(const void *key_p, const void *parameters);  
            /* Returns: secondary hash value for *key_p             */
            /* Note: return unity for linear probing                */
} hash_class_t;


/************************************************************
 * SELECTORS
 ************************************************************/

#define hash_class_minload(class_p)            ((class_p)->minload)
#define hash_class_maxload(class_p)            ((class_p)->maxload)
#define hash_class_maximum_capacity(class_p)   \
        ((class_p)->capacity[(class_p)->capacity_count-1])

#define hash_class_collisions(class_p)    ((class_p)->collisions)
        /* Returns: number of collisions across all tables in class   */
#define hash_class_redundancies(class_p)  ((class_p)->redundancies)
        /* Returns: number of redundancies across all tables in class */
#define hash_class_probes(class_p)        ((class_p)->probes)
        /* Returns: number of probes across all tables in class       */
#define hash_class_mean_collisions(class_p)        \
        ((hash_class_probes(class_p) == 0) ? 0.0 : \
         (hash_class_collisions(class_p) / hash_class_probes(class_p)))
        /* Returns: mean number of collisions per probe for class     */
#define hash_class_mean_redundancies(class_p)      \
        ((hash_class_probes(class_p) == 0) ? 0.0 : \
         (hash_class_redundancies(class_p) / hash_class_probes(class_p)))
        /* Returns: mean number of redundancies per probe for class   */

#endif

