/*
 * (c) 1997 Eric Sven Ristad.  All rights reserved.
 *
 * $Id: logpr.h,v 1.3 1997/12/01 00:39:09 libpa Exp $
 *
 */

#ifndef LOGPR_INCLUDED
#define LOGPR_INCLUDED
#include <float.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
extern double log(double);
extern double exp(double);
extern int    finite(double);
extern int    kill(pid_t, int);
extern pid_t  getpid(void);
#ifndef M_LN2
#define M_LN2          6.9314718055994530942E-1
#endif
#ifndef DBL_INFINITY
#define DBL_INFINITY  (DBL_MAX*2)
#endif


/* Description.
 *  Logarithmic representation of probability values with
 *   double precision floating-point numbers.
 */

typedef double logpr_t;

#define LOGPR_ZERO             (-DBL_INFINITY)
#define LOGPR_UNITY             0.0
#define LOGPR_SIGNAL            SIGFPE


/************************************************************
 * Constructors and Coercion
 ************************************************************/

#define logpr2double(l)         (((l) <= (DBL_MIN_EXP*M_LN2)) ? 0 : exp(l))
#define double2logpr(x)         (((0 < (x)) && ((x) <= 1)) ? log(x) : \
				 (((x) == 0) ? LOGPR_ZERO : logpr_raise()))

#define logpr2nats(l)           (-(l))
#define nats2logpr(n)           (-(n))

#define logpr2bits(l)           (-(l) / M_LN2)
#define bits2logpr(b)           (-(b) * M_LN2)

/************************************************************
 * Predicates
 ************************************************************/

#define logpr_is_valid(lx)      ((lx) <= 0)
#define logpr_compare(lx,ly)    (((lx) < (ly)) ? -1 : (((lx) > (ly)) ? 1 : 0))
#define logpr_are_equal(lx,ly)  ((lx) == (ly))
#define logpr_is_greater(lx,ly) ((lx) > (ly))
#define logpr_is_less(lx,ly)    ((lx) < (ly))
#define logpr_is_geq(lx,ly)     ((lx) >= (ly))
#define logpr_is_leq(lx,ly)     ((lx) <= (ly))
#define logpr_is_zero(lx)       (!finite(lx))
#define logpr_is_positive(lx)   (finite(lx))

/************************************************************
 * Arithmetic
 ************************************************************/

#ifndef NDEBUG

 #define logpr_multiply(lx,ly)   ((finite(lx) && finite(ly)) ? ((lx)+(ly)) : LOGPR_ZERO)
 #define logpr_divide(lx,ly)     (!finite(lx) ? LOGPR_ZERO : \
				  (((lx) <= (ly)) ? ((lx)-(ly)) : logpr_raise()))
 #define logpr_complement(lx)    (((lx) < 0) ? double2logpr(1-logpr2double(lx)) : LOGPR_ZERO)
 logpr_t logpr_add(logpr_t lx, logpr_t ly);

#else

 #define logpr_multiply(lx,ly)   ((lx)+(ly))
 #define logpr_divide(lx,ly)     ((lx)-(ly))
 #define logpr_complement(lx)    (((lx) < 0) ? double2logpr(1-logpr2double(lx)) : LOGPR_ZERO)
 logpr_t logpr_add(logpr_t lx, logpr_t ly);

#endif

/************************************************************
 * Exception Handling
 ************************************************************/

#define logpr_raise()           (kill(getpid(), LOGPR_SIGNAL))

#endif

