/*
 * (c) 1996,1997 Eric Sven Ristad and Peter N. Yianilos.  All rights reserved.
 *
 * $Id: memory.spartan.h,v 1.4 1997/02/11 20:33:34 ristad Exp $
 *
 */

#ifndef MEMORY_SPARTAN_INCLUDED
#define MEMORY_SPARTAN_INCLUDED
#include "common.h"  /* for boolean_t */
#include "memory.h"

/* Description.
 *  Simple interface to machine memory without any memory overhead.
 *  Spartan blocks are of unknown size and do not have any caller workspace.
 *  Therefore, all spartan operations require knowledge of memory block size, 
 *  and hence are most suitable for allocating objects of known fixed size,
 *  such as structs and primitive C types.  
 *
 *  WARNING: Mixing regular and spartan calls is extremely dangerous!
 *           Spartan calls should not be used except in dire emergency.
 *
 */

void     *memory_spartan_create(unsigned bytes);
          /* Returns: address of memory block of specified size   */

void      memory_spartan_destroy(void *block, unsigned bytes);
          /* Requires: block returned by a spartan memory_t operation */
          /* Effect: return specified block of memory to system       */

void     *memory_spartan_recreate(void *block, unsigned old_bytes, unsigned new_bytes);
          /* Requires: block was returned by a spartan memory_t operation   */
          /* Effect: destroys original block if new block is allocated      */
          /* Returns: block of storage that is exact copy of size new_bytes */

void     *memory_spartan_copy(const void *block, unsigned bytes);
          /* Requires: block returned by spartan memory_t operation    */
          /* Returns: new block of storage that is exact copy of block */
          /* Warning: this is not equivalent to memcpy() in string.h   */

boolean_t memory_spartan_address_is_valid(const void *block, unsigned bytes, 
					  const void *address);
          /* Returns: TRUE iff address falls within spartan block    */

int       memory_spartan_compare(const void *left, unsigned left_bytes,
				 const void *right, unsigned right_bytes);
          /* Returns: 0 iff both blocks of memory are byte-wise equivalent */


#endif	/* MEMORY_SPARTAN_INCLUDED */








