/*
 * (c) 1996 Eric Sven Ristad.  All rights reserved.
 *
 * $Id: suffix.internal.h,v 1.8 1998/02/07 00:40:44 libpa Exp $
 *
 */

#ifndef SUFFIX_INTERNAL_INCLUDED
#define SUFFIX_INTERNAL_INCLUDED
#include "vector.h"
#include "suffix.h"

/* DESCRIPTION
 *  Internal types and operations for suffixes and their indices.
 *  These types and prototypes should not be exported.
 */

typedef struct bounded_corpus_tag
{
  unsigned        bound;
  const vector_t *corpus;
} bounded_corpus_t;

/************************************************************
 * primitive (suffix_t *) and suffix_index_t operations
 ************************************************************/

#define   suffix_length(suffix,corpus)    \
          ((unsigned)((suffix_t *)(suffix) - (suffix_t *)corpus))
#define   suffix2index(suffix,corpus)     \
          ((suffix_length(suffix,corpus)/vector_object_size(corpus)) - 1)
#define   suffix_is_valid(suffix,corpus)  \
          (vector_index_is_valid(corpus, suffix2index(suffix,corpus)))

#define   suffix_index_is_valid(index,corpus)  (vector_index_is_valid((corpus),(index)))
#define   suffix_index2length(index,corpus)    (((index)+1)*vector_object_size(corpus))
#define   suffix_index2suffix(index,corpus)    \
          (((suffix_t *)(corpus))+suffix_index2length(index,corpus))

/************************************************************
 * primitive (suffix_t *) comparison
 ************************************************************/

int       suffix_compare(unsigned, const suffix_t *, unsigned, const suffix_t *);
          /* Returns: total order on suffixes, shorter < longer for inclusion */
boolean_t suffix_includes(unsigned, const suffix_t *, unsigned, const suffix_t *);
          /* Returns: TRUE iff first suffix includes second affix */
unsigned  suffix_common_length(unsigned, const suffix_t *, unsigned, const suffix_t *);
          /* Returns: number of bytes both suffixes have in common */

/************************************************************
 * (suffix_index_t *) comparison for use in vector_sorted
 ************************************************************/

int       suffix_index_compare(const void *, const void *, const vector_t *corpus);
          /* Returns: comparison between two (suffix_index_t *)               */
void      suffix_indices_sort(suffix_index_t *indices, const vector_t *corpus);
          /* Effect: sorts suffix indices wrt corpus using quicksort */
void      suffix_indices_byte_sort(suffix_index_t *indices, const vector_t *corpus, 
				   unsigned span_limit);
          /* Effect: sorts suffix indices wrt corpus using df radix sort until */
          /*         indices span falls below span_limit, then quicksort       */
boolean_t suffix_indices_are_sorted(suffix_index_t *indices, const vector_t *corpus);
          /* Returns: TRUE iff suffix indices are sorted wrt corpus*/

int       uint_compare(const void *left_p, const void *right_p, const void *);
          /* Returns: literal comparison between indices as unsigned integers */

/************************************************************
 * bounded (suffix_index_t *) comparison 
 ************************************************************/

int       suffix_index_bounded_compare(const void *, const void *, const void *enclosure);
          /* Returns: bounded comparison between two (suffix_index_t *)       */
void      suffix_indices_bounded_sort(suffix_index_t *indices, 
				      const vector_t *corpus, unsigned depth);
          /* Effect: sorts suffix indices to specified depth using quicksort */
void      suffix_indices_bounded_byte_sort(suffix_index_t *indices, 
					   const vector_t *corpus, unsigned depth, 
					   unsigned span_limit);
          /* Effect: sorts suffix indices wrt corpus using df radix sort */
boolean_t suffix_indices_are_bounded_sorted(const suffix_index_t *indices, 
					    const vector_t *corpus, unsigned depth);
          /* Returns: TRUE iff indices are sorted to specified depth */



#endif



