/*
 * (c) 1995 Eric Sven Ristad.  All rights reserved.
 * 
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "memory.h"
#include "vector.h"
#include "table.h"
#include "unigram.h"

static char unigram_bench_rcsid[] = "$Id: unigram.bench.c,v 1.2 1997/06/05 22:09:45 ristad Exp $";

#define MAX_SYMBOL            UCHAR_MAX

int main(int argc, char **argv)
{
  FILE      *stream;
  char      *file;
  int        mode, byte;

  if (argc != 3)
    {
      printf("Usage:\t%s [0-2] file\n", argv[0]);
      return(1);
    }

  mode = atoi(argv[1]);
  file = argv[2];

  stream = fopen(file, "r");
  if (stream == NULL)
    {
      printf("Error: %s could not open file %s\n", argv[0], file);
      return(1);
    }

  switch(mode)
    {
    case 0:
      {
	unsigned i, q, n, *vector;

	vector = vector_create(sizeof(unsigned), MAX_SYMBOL+1);
	assert(vector);
	q = 0;
	n = 0;
	for(i = 0; i <= MAX_SYMBOL; i++) { vector[i] = 0; }
	while((byte = fgetc(stream)) != EOF)
	  {
	    if (vector[byte] == 0) { q++; }
	    vector[byte]++;
	    n++;
	  }
	printf("Unigram vector_t benchmark (q=%u, n=%u, %u bytes).\n", 
	       q, n, memory_total_bytes()+(2*sizeof(int)));
	vector_destroy(vector);
      }
      break;
    case 1:
      {
	table_t *table;
	unsigned char symbol;
	unsigned n, unity, *count_p;

	n = 0;
	unity = 1;
	table = table_create(sizeof(char), sizeof(unsigned));
	assert(table);
	while((byte = fgetc(stream)) != EOF)
	  {
	    n++;
	    symbol = byte;
	    count_p = table_lookup(table, &symbol);
	    if (count_p == NULL) { table_insert(table, &symbol, &unity); }
	    else { *count_p++; }
	  }
	printf("Unigram table_t benchmark (q=%u, n=%u, %u bytes).\n", 
	       table_size(table), n, memory_total_bytes()+sizeof(int));
	table_destroy(table);
      }
      break;
    case 2:
      {
	unigram_t *unigram;
	unsigned char symbol;

	unigram = unigram_create(sizeof(char));
	assert(unigram);
	while((byte = fgetc(stream)) != EOF)
	  {
	    symbol = byte;
	    unigram_increment_frequency(unigram, &symbol);
	  }
	printf("Unigram unigram_t benchmark (q=%u, n=%u, %u bytes).\n", 
	       unigram_diversity(unigram), unigram_trials(unigram), 
	       memory_total_bytes());
	unigram_destroy(unigram);
      }
      break;
    default:
      printf("Usage:\t%s [0-2] file\n", argv[0]);
      return(1);
    }

  fclose(stream);
  assert(memory_total_bytes() == 0);
  assert(memory_total_blocks() == 0);
  return(0);
}



