/*
 * (c) 1998 Eric Sven Ristad.  All rights reserved.
 *
 * $Id: unigram.h,v 1.5 1998/02/07 03:46:35 libpa Exp $
 *
 */

#ifndef UNIGRAM_INCLUDED
#define UNIGRAM_INCLUDED

#include "common.h"
#include "table.h"

/* Description.
 *  Compact table for storing symbol frequencies, where the
 *  symbol counters are dynamically resized to store largest frequency.
 *
 */

typedef table_t unigram_t;

/************************************************************
 * Constructors/Destructors
 ************************************************************/

unigram_t *unigram_create(unsigned symbol_size);
           /* Returns: allocates, initializes, and returns unigram_t */
void       unigram_destroy(unigram_t *unigram);
           /* Returns: finalizes and deallocates unigram_t           */

void       unigram_initialize(unigram_t *unigram, unsigned symbol_size);
           /* Returns: initializes caller-allocated unigram_t */
void       unigram_finalize(unigram_t *unigram);
           /* Returns: finalizes caller-allocated unigram_t   */

/************************************************************
 * Selectors
 ************************************************************/

#define    unigram_diversity(uni_p)        (table_size((table_t *)(uni_p)))
           /* Returns: number of symbols observed at least once  */
#define    unigram_counter_size(uni_p)     (table_datum_size((table_t *)(uni_p)))
           /* Returns: number of bytes used to store counters    */
#define    unigram_symbol_size(uni_p)      (table_key_size((table_t *)(uni_p)))
           /* Returns: number of bytes used to represent symbols */
#define    unigram_trials(uni_p)           (((table_t *)(uni_p))->workspace)
           /* Returns: unsigned sum of all symbol frequencies    */

boolean_t  unigram_is_valid(const unigram_t *unigram);
           /* Returns: TRUE iff unigram is self-consistent */
unsigned   unigram_maximum(const unigram_t *unigram);
           /* Returns: maximum symbol frequency in O(n) time */
unsigned   unigram_frequency(const unigram_t *unigram, const void *symbol_p);
           /* Returns: frequency of given *symbol_p        */

/************************************************************
 * Mutators
 ************************************************************/

void       unigram_set_frequency(unigram_t *unigram, const void *symbol_p, 
				 unsigned frequency);
           /* Effect: sets frequency of given *symbol_p       */
void       unigram_increment_frequency(unigram_t *unigram, const void *symbol_p);
           /* Effect: increments frequency of given *symbol_p */
void       unigram_clear(unigram_t *unigram);
           /* Effect: zeros all symbol frequencies            */

/************************************************************
 * Higher Order Procedure
 ************************************************************/

void       unigram_map(const unigram_t *unigram, 
		       boolean_t (*proc)(const void *symbol_p, unsigned frequency, 
					 void *parameters),
		       void *parameters);
           /* Effect: call proc() on all <symbol,frequency> pairs while TRUE */

#endif
